/*
 * Thread-related initialisation activities.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/c/util/cap_alloc.h>
#include <l4/re/env.h>
#include <l4/sys/factory.h>
#include <l4/sys/semaphore.h>

#include "cap_alloc.h"
#include "semaphore.h"
#include "thread.h"



/* Bootstrap the semaphore support needed by the allocation function. */

static long ipc_semaphore_init(l4_cap_idx_t *semaphore)
{
  if (l4_is_invalid_cap(*semaphore))
  {
    *semaphore = l4re_util_cap_alloc();

    if (l4_is_invalid_cap(*semaphore))
      return -L4_ENOENT;

    if (l4_error(l4_factory_create(l4re_env()->factory, L4_PROTO_SEMAPHORE, *semaphore)))
    {
      ipc_cap_free_um(*semaphore);
      return -L4_ENOENT;
    }

    return ipc_semaphore_up(*semaphore);
  }
  else
    return L4_EOK;
}

/* Bootstrap the semaphore support needed by the allocation functions. */

extern l4_cap_idx_t ipc_cap_alloc_semaphore;

long ipc_cap_alloc_init()
{
  return ipc_semaphore_init(&ipc_cap_alloc_semaphore);
}

/* Bootstrap the semaphore support needed by the memory functions. */

extern l4_cap_idx_t ipc_mem_semaphore;

long ipc_mem_init()
{
  return ipc_semaphore_init(&ipc_mem_semaphore);
}

/* General semaphore initialisation. */

long ipc_thread_init()
{
  long err;

  err = ipc_cap_alloc_init();
  if (err)
    return err;

  return ipc_mem_init();
}

// vim: tabstop=2 expandtab shiftwidth=2
